@@ -1,6 +1,6 @@
 /*
  * JBoss, Home of Professional Open Source.
- * Copyright 2012, Red Hat, Inc., and individual contributors
+ * Copyright 2016, Red Hat, Inc., and individual contributors
  * as indicated by the @author tags. See the copyright.txt file in the
  * distribution for a full listing of individual contributors.
  *
@@ -22,40 +22,49 @@
 
 package org.jboss.logging.processor.apt;
 
-import static java.util.Collections.unmodifiableSet;
-import static org.jboss.logging.processor.util.ElementHelper.findByName;
-import static org.jboss.logging.processor.util.ElementHelper.inheritsMessage;
-import static org.jboss.logging.processor.util.ElementHelper.isOverloaded;
-import static org.jboss.logging.processor.util.ElementHelper.parameterCount;
+import static org.jboss.logging.processor.util.ElementHelper.isAnnotatedWith;
 import static org.jboss.logging.processor.util.Objects.HashCodeBuilder;
 import static org.jboss.logging.processor.util.Objects.ToStringBuilder;
 import static org.jboss.logging.processor.util.Objects.areEqual;
 
+import java.lang.annotation.Annotation;
+import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
-import java.util.EnumMap;
+import java.util.HashMap;
 import java.util.Iterator;
 import java.util.LinkedHashSet;
 import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
+import java.util.Properties;
 import java.util.Set;
+import javax.annotation.processing.ProcessingEnvironment;
+import javax.lang.model.element.AnnotationMirror;
 import javax.lang.model.element.ExecutableElement;
+import javax.lang.model.element.Name;
+import javax.lang.model.element.TypeElement;
+import javax.lang.model.element.VariableElement;
 import javax.lang.model.type.TypeMirror;
 import javax.lang.model.util.Elements;
-import javax.lang.model.util.Types;
 
 import org.jboss.logging.Logger;
+import org.jboss.logging.annotations.Cause;
+import org.jboss.logging.annotations.Field;
 import org.jboss.logging.annotations.LogMessage;
 import org.jboss.logging.annotations.Message;
 import org.jboss.logging.annotations.Message.Format;
+import org.jboss.logging.annotations.Param;
+import org.jboss.logging.annotations.Pos;
+import org.jboss.logging.annotations.Property;
+import org.jboss.logging.annotations.Suppressed;
 import org.jboss.logging.processor.model.MessageMethod;
 import org.jboss.logging.processor.model.Parameter;
-import org.jboss.logging.processor.model.Parameter.ParameterType;
 import org.jboss.logging.processor.model.ReturnType;
 import org.jboss.logging.processor.model.ThrowableType;
 import org.jboss.logging.processor.util.Comparison;
 import org.jboss.logging.processor.util.ElementHelper;
+import org.jboss.logging.processor.util.Expressions;
 
 /**
  * Date: 29.07.2011
@@ -66,21 +75,22 @@ final class MessageMethodBuilder {
 
     private static final String MESSAGE_METHOD_SUFFIX = "$str";
     private final List<ExecutableElement> methods;
-    private final Elements elements;
-    private final Types types;
+    private final ProcessingEnvironment processingEnv;
+    private final Properties expressionProperties;
 
-    private MessageMethodBuilder(final Elements elements, final Types types) {
-        this.elements = elements;
-        this.types = types;
+    private MessageMethodBuilder(final ProcessingEnvironment processingEnv, final Properties expressionProperties) {
+        this.processingEnv = processingEnv;
+        this.expressionProperties = expressionProperties;
         methods = new LinkedList<>();
     }
 
-    MessageMethodBuilder add(final ExecutableElement method) {
-        methods.add(method);
+    MessageMethodBuilder add(final Collection<ExecutableElement> methods) {
+        this.methods.addAll(methods);
         return this;
     }
 
     Set<MessageMethod> build() {
+        final Elements elements = processingEnv.getElementUtils();
         final Set<MessageMethod> result = new LinkedHashSet<>();
         for (ExecutableElement elementMethod : methods) {
             final AptMessageMethod resultMethod = new AptMessageMethod(elements, elementMethod);
@@ -88,11 +98,11 @@ final class MessageMethodBuilder {
             resultMethod.message = findMessage(methods, elementMethod);
             resultMethod.isOverloaded = isOverloaded(methods, elementMethod);
             for (TypeMirror thrownType : elementMethod.getThrownTypes()) {
-                resultMethod.thrownTypes.add(ThrowableTypeFactory.of(elements, types, thrownType));
+                resultMethod.thrownTypes.add(ThrowableTypeFactory.of(processingEnv, thrownType));
             }
 
             // Create a list of parameters
-            for (Parameter parameter : ParameterFactory.of(elements, types, resultMethod.method)) {
+            for (Parameter parameter : ParameterFactory.of(processingEnv, resultMethod.method)) {
                 resultMethod.add(parameter);
             }
             // Check to see if the method is overloaded
@@ -104,7 +114,7 @@ final class MessageMethodBuilder {
                 resultMethod.translationKey = resultMethod.name();
             }
             // Set the return type
-            resultMethod.returnType = ReturnTypeFactory.of(elements, types, elementMethod.getReturnType(), resultMethod);
+            resultMethod.returnType = ReturnTypeFactory.of(processingEnv, elementMethod.getReturnType(), resultMethod);
             result.add(resultMethod);
         }
         return Collections.unmodifiableSet(result);
@@ -114,7 +124,7 @@ final class MessageMethodBuilder {
         AptMessage result = null;
         Message message = method.getAnnotation(Message.class);
         if (message != null) {
-            result = new AptMessage(message);
+            result = new AptMessage(message, expressionProperties);
             result.hasId = hasMessageId(message);
             result.inheritsId = message.id() == Message.INHERIT;
             if (result.inheritsId()) {
@@ -126,11 +136,11 @@ final class MessageMethodBuilder {
                 result.id = message.id();
             }
         } else {
-            final Collection<ExecutableElement> allMethods = findByName(methods, method.getSimpleName(), parameterCount(method.getParameters()));
+            final Collection<ExecutableElement> allMethods = findByName(methods, method);
             for (ExecutableElement m : allMethods) {
                 message = m.getAnnotation(Message.class);
                 if (message != null) {
-                    result = new AptMessage(message);
+                    result = new AptMessage(message, expressionProperties);
                     result.hasId = hasMessageId(message);
                     result.inheritsId = message.id() == Message.INHERIT;
                     if (result.inheritsId()) {
@@ -166,8 +176,110 @@ final class MessageMethodBuilder {
         return message != null && (message.id() != Message.NONE && message.id() != Message.INHERIT);
     }
 
-    static MessageMethodBuilder create(final Elements elements, final Types types) {
-        return new MessageMethodBuilder(elements, types);
+    private Collection<ExecutableElement> findByName(final Collection<ExecutableElement> methods, final ExecutableElement method) {
+        final Name methodName = method.getSimpleName();
+        final List<ExecutableElement> result = new ArrayList<>();
+        final int paramCount = parameterCount(method.getParameters());
+        for (ExecutableElement m : methods) {
+            if (methodName.equals(m.getSimpleName()) && parameterCount(m.getParameters()) == paramCount) {
+                result.add(m);
+            }
+        }
+        return result;
+    }
+
+
+    /**
+     * Returns a collection of methods with the same name.
+     *
+     * @param methods    the methods to process.
+     * @param methodName the method name to findByName.
+     *
+     * @return a collection of methods with the same name.
+     */
+    private Collection<ExecutableElement> findByName(final Collection<ExecutableElement> methods, final Name methodName) {
+        final List<ExecutableElement> result = new ArrayList<>();
+        for (ExecutableElement method : methods) {
+            if (methodName.equals(method.getSimpleName())) {
+                result.add(method);
+            }
+        }
+        return result;
+    }
+
+    /**
+     * Checks to see if the method has or inherits a {@link org.jboss.logging.annotations.Message}
+     * annotation.
+     *
+     * @param methods the method to search.
+     * @param method  the method to check.
+     *
+     * @return {@code true} if the method has or inherits a message annotation, otherwise {@code false}.
+     */
+
+    private boolean inheritsMessage(final Collection<ExecutableElement> methods, final ExecutableElement method) {
+        if (isAnnotatedWith(method, Message.class)) {
+            return false;
+        }
+        final Collection<ExecutableElement> allMethods = findByName(methods, method.getSimpleName());
+        for (ExecutableElement m : allMethods) {
+            if (isAnnotatedWith(m, Message.class)) {
+                return true;
+            }
+        }
+        return false;
+    }
+
+    /**
+     * Checks to see if the method is overloaded. An overloaded method has a different parameter count based on the
+     * format parameters only. Parameters annotated with {@link org.jboss.logging.annotations.Cause} or
+     * {@link org.jboss.logging.annotations.Param}
+     * are not counted.
+     *
+     * @param methods the method to search.
+     * @param method  the method to check.
+     *
+     * @return {@code true} if the method is overloaded, otherwise {@code false}.
+     */
+    private boolean isOverloaded(final Collection<ExecutableElement> methods, final ExecutableElement method) {
+        final Collection<ExecutableElement> allMethods = findByName(methods, method.getSimpleName());
+        for (ExecutableElement m : allMethods) {
+            if (method.getSimpleName().equals(m.getSimpleName()) && parameterCount(method.getParameters()) != parameterCount(m.getParameters())) {
+                return true;
+            }
+        }
+        return false;
+    }
+
+    /**
+     * Returns the number of parameters excluding the {@link org.jboss.logging.annotations.Cause} parameter
+     * and any {@link org.jboss.logging.annotations.Param} parameters if found.
+     *
+     * @param params the parameters to get the count for.
+     *
+     * @return the number of parameters.
+     */
+    private int parameterCount(final Collection<? extends VariableElement> params) {
+        int result = params.size();
+        boolean hasCause = false;
+        for (VariableElement param : params) {
+            if (isAnnotatedWith(param, Param.class) || isAnnotatedWith(param, Field.class) ||
+                    isAnnotatedWith(param, Property.class) || isAnnotatedWith(param, Suppressed.class)) {
+                --result;
+            }
+            if (isAnnotatedWith(param, Cause.class)) {
+                hasCause = true;
+            }
+        }
+        return (result - (hasCause ? 1 : 0));
+    }
+
+    static MessageMethodBuilder create(final ProcessingEnvironment processingEnv) {
+        return create(processingEnv, new Properties());
+    }
+
+    static MessageMethodBuilder create(final ProcessingEnvironment processingEnv, final Properties expressionProperties) {
+        return new MessageMethodBuilder(processingEnv, expressionProperties);
     }
 
     /**
@@ -176,7 +288,7 @@ final class MessageMethodBuilder {
     private static class AptMessageMethod implements MessageMethod {
 
         private final Elements elements;
-        private final Map<ParameterType, Set<Parameter>> parameters;
+        private final Map<TypeMirror, Set<Parameter>> parameters;
         private final Set<ThrowableType> thrownTypes;
         private final ExecutableElement method;
         private ReturnType returnType;
@@ -186,6 +298,7 @@ final class MessageMethodBuilder {
         private Message message;
         private String messageMethodName;
         private String translationKey;
+        private int formatParameterCount;
 
         /**
          * Private constructor for the
@@ -198,26 +311,36 @@ final class MessageMethodBuilder {
             this.method = method;
             inheritsMessage = false;
             isOverloaded = false;
-            parameters = new EnumMap<>(ParameterType.class);
+            parameters = new HashMap<>();
             thrownTypes = new LinkedHashSet<>();
+            formatParameterCount = 0;
         }
 
         void add(final Parameter parameter) {
-            if (parameters.containsKey(ParameterType.ANY)) {
-                parameters.get(ParameterType.ANY).add(parameter);
+            if (parameter.isFormatParameter()) {
+                if (parameter.isAnnotatedWith(Pos.class)) {
+                    formatParameterCount += parameter.getAnnotation(Pos.class).value().length;
+                } else {
+                    formatParameterCount++;
+                }
+            }
+            if (parameters.containsKey(null)) {
+                parameters.get(null).add(parameter);
             } else {
                 final Set<Parameter> any = new LinkedHashSet<>();
                 any.add(parameter);
-                parameters.put(ParameterType.ANY, any);
+                parameters.put(null, any);
             }
-            if (parameters.containsKey(parameter.parameterType())) {
-                parameters.get(parameter.parameterType()).add(parameter);
-            } else {
-                final Set<Parameter> set = new LinkedHashSet<>();
-                set.add(parameter);
-                parameters.put(parameter.parameterType(), set);
+            for (AnnotationMirror a : parameter.getAnnotationMirrors()) {
+                if (parameters.containsKey(a.getAnnotationType())) {
+                    parameters.get(a.getAnnotationType()).add(parameter);
+                } else {
+                    final Set<Parameter> set = new LinkedHashSet<>();
+                    set.add(parameter);
+                    parameters.put(a.getAnnotationType(), set);
+                }
             }
-            if (parameter.parameterType() == ParameterType.CAUSE) {
+            if (parameter.isAnnotatedWith(Cause.class)) {
                 cause = parameter;
             }
         }
@@ -228,25 +351,17 @@ final class MessageMethodBuilder {
         }
 
         @Override
-        public Set<Parameter> parameters(final ParameterType parameterType) {
-            if (parameters.containsKey(parameterType)) {
-                return parameters.get(parameterType);
+        public Set<Parameter> parameters() {
+            if (parameters.containsKey(null)) {
+                return Collections.unmodifiableSet(parameters.get(null));
             }
             return Collections.emptySet();
         }
 
         @Override
-        public Set<Parameter> parameters(final ParameterType parameterType, final ParameterType... parameterTypes) {
-            final Set<Parameter> result = new LinkedHashSet<Parameter>();
-            if (parameters.containsKey(parameterType)) {
-                result.addAll(parameters.get(parameterType));
-            }
-            for (ParameterType pt : parameterTypes) {
-                if (parameters.containsKey(pt)) {
-                    result.addAll(parameters.get(pt));
-                }
-            }
-            return result;
+        public Set<Parameter> parametersAnnotatedWith(final Class<? extends Annotation> annotation) {
+            final TypeElement type = ElementHelper.toTypeElement(elements, annotation);
+            return parameters.containsKey(type.asType()) ? Collections.unmodifiableSet(parameters.get(type.asType())) : Collections.emptySet();
         }
 
         @Override
@@ -256,7 +371,7 @@ final class MessageMethodBuilder {
 
         @Override
         public Set<ThrowableType> thrownTypes() {
-            return unmodifiableSet(thrownTypes);
+            return Collections.unmodifiableSet(thrownTypes);
         }
 
         @Override
@@ -311,34 +426,33 @@ final class MessageMethodBuilder {
 
         @Override
         public String logLevel() {
-            // TODO (jrp) possibly return the actual level
             final LogMessage logMessage = method.getAnnotation(LogMessage.class);
-            final Logger.Level logLevel = (logMessage.level() == null ? Logger.Level.INFO : logMessage.level());
-            return String.format("%s.%s.%s", Logger.class.getName(), Logger.Level.class.getSimpleName(), logLevel.name());
+            return (logMessage.level() == null ? Logger.Level.INFO.name() : logMessage.level().name());
         }
 
         @Override
         public int formatParameterCount() {
-            int result = parameters(ParameterType.FORMAT, ParameterType.TRANSFORM).size();
-            for (Parameter params : parameters(ParameterType.POS)) {
-                result += params.pos().value().length;
-            }
-            return result;
+            return formatParameterCount;
         }
 
         @Override
         public boolean isLoggerMethod() {
-            return ElementHelper.isAnnotatedWith(method, LogMessage.class);
+            return isAnnotatedWith(LogMessage.class);
         }
 
         @Override
         public int hashCode() {
             return HashCodeBuilder.builder()
                     .add(name())
-                    .add(parameters(ParameterType.ANY))
+                    .add(parameters())
                     .add(returnType()).toHashCode();
         }
 
+        @Override
+        public ExecutableElement getDelegate() {
+            return method;
+        }
+
         @Override
         public boolean equals(final Object obj) {
             if (obj == this) {
@@ -358,25 +472,20 @@ final class MessageMethodBuilder {
             return ToStringBuilder.of(this)
                     .add("name", name())
                     .add("returnType", returnType())
-                    .add("parameters", parameters(ParameterType.ANY))
+                    .add("parameters", parameters())
                     .add("loggerMethod", loggerMethod()).toString();
         }
 
-        @Override
-        public ExecutableElement reference() {
-            return method;
-        }
-
         @Override
         public int compareTo(final MessageMethod o) {
             int result = name().compareTo(o.name());
             result = (result != Comparison.EQUAL) ? result : returnType.name().compareTo(o.returnType().name());
             // Size does matter
-            result = (result != Comparison.EQUAL) ? result : parameters(ParameterType.ANY).size() - o.parameters(ParameterType.ANY).size();
+            result = (result != Comparison.EQUAL) ? result : parameters().size() - o.parameters().size();
             if (result == Comparison.EQUAL) {
                 // Check element by element
-                final Iterator<Parameter> params1 = parameters(ParameterType.ANY).iterator();
-                final Iterator<Parameter> params2 = o.parameters(ParameterType.ANY).iterator();
+                final Iterator<Parameter> params1 = parameters().iterator();
+                final Iterator<Parameter> params2 = o.parameters().iterator();
                 while (params1.hasNext()) {
                     if (params2.hasNext()) {
                         final Parameter param1 = params1.next();
@@ -401,12 +510,18 @@ final class MessageMethodBuilder {
     private static class AptMessage implements MessageMethod.Message {
 
         private final Message message;
+        private final String messageValue;
         private int id;
         private boolean hasId;
         private boolean inheritsId;
 
-        private AptMessage(final Message message) {
+        private AptMessage(final Message message, final Properties expressionProperties) {
             this.message = message;
+            if (expressionProperties.isEmpty()) {
+                this.messageValue = message.value();
+            } else {
+                this.messageValue = Expressions.resolve(expressionProperties, message.value());
+            }
         }
 
         @Override
@@ -426,7 +541,7 @@ final class MessageMethodBuilder {
 
         @Override
         public String value() {
-            return message.value();
+            return messageValue;
         }
 
         @Override
